//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "knownfilelist.h"
#include "opcodes.h"
#include "emule.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CKnownFileList::CKnownFileList(const CString& in_appdir)
{	
	filesFilePath.Format("%sknown.met", in_appdir);	
	trafficFilePath.Format("%straffic.dat", in_appdir);
	partPermissionFilePath.Format("%spartperm.dat", in_appdir);
	accepted = 0;
	requested = 0;
	transfered = 0;
	m_bInSave	=	false;
	//--- xrmb:statsautosave ---
	//lastsaved=GetTickCount();
	//--- :xrmb ---
	Init();
}

CKnownFileList::~CKnownFileList(){
	Clear();
}


bool CKnownFileList::Init()
{
	EMULE_TRY

	CKnownFile* Record = 0; 
	CSafeFile file;
	try {		
		if (!file.Open(filesFilePath,CFile::modeRead|CFile::osSequentialScan)){			
			return false;
		}		
		uint8 header;
		file.Read(&header,1);
		if (header != MET_HEADER){
			file.Close();
			return false;
		}
		CSingleLock sLock(&list_mut,true); // to make sure that its thread-safe
		uint32 RecordsNumber;
		file.Read(&RecordsNumber,4);
		for (uint32 i = 0; i < RecordsNumber; i++) {
			Record =  new CKnownFile();
			if ( Record )
			{
				if ( Record->LoadFromFile(&file) )
				{
					Add(Record);
				}
				else
				{
					delete Record;
				}
			}
		}

		//--- xrmb:parttraffic ---
		file.Close();
		LoadPartTraffic();
		//sLock.Unlock();
		//--- :xrmb ---

		//--- xrmb:partprio ---
		//--- restore the hidden/blocked settings ---
		LoadPartPrio();
		sLock.Unlock();
		//--- :xrmb ---

		return true;
	}
	catch(CFileException* error){
		OUTPUT_DEBUG_TRACE();
		if (error->m_cause == CFileException::endOfFile)
			AddLogLine(true,GetResString(IDS_ERR_SERVERMET_BAD));
		else{
			char buffer[150];
			error->GetErrorMessage(buffer,150);
			AddLogLine(true,GetResString(IDS_ERR_SERVERMET_UNKNOWN),buffer);
		}
		error->Delete();	//memleak fix
		return false;
	}

	EMULE_CATCH

	return false;
}

// Tarod - Now thread-safe
void CKnownFileList::Save()
{
	EMULE_TRY

	// We need to do it thread safe because now is called under CUploadQueue::RemoveFromUploadQueue()
	// Also no called from separate thread each 30 min
	CSingleLock sLock(&list_mut, true);

	// to avoid situation when sources saved both  each 30 min and on exit
	if ( m_bInSave )
	{
		return;
	}

	m_bInSave	=	true;

	FILE* file = NULL; 	
	if (!(file = fopen(filesFilePath, "wb")))
	{		
		return;
	}	
  
	fputc(MET_HEADER,file);
	uint32 RecordsNumber = GetCount();
	fwrite(&RecordsNumber,4,1,file); 
  
	// Old write routine
	//for (uint32 i = 0; i < RecordsNumber; i++) {
	// ElementAt(i)->WriteToFile(file);
	//} 
  
	// This routine was changed to take in account possible NULL pointers
	// I think is not needed anymore because now is thread safe
	uint32 i = 0;
	while (i < RecordsNumber)
	{
		if (ElementAt(i) == NULL) // if is NULL is unuseful so we remove it!
		{
			RecordsNumber--;
			RemoveAt(i);
		} else { // element is not NULL, write it an go for next one
			((CKnownFile*)ElementAt(i))->WriteToFile(file);
			i++;
		}
	} 
  
	// Maybe we have less records than before, rewrite number of records
	fseek(file, 1L, SEEK_SET);
	fwrite(&RecordsNumber, 4, 1, file); 
  
	sLock.Unlock(); // free mutex before closing 
	fclose(file); 

	//--- xrmb:parttraffic ---
	SavePartTraffic();
	//--- :xrmb ---

	//--- xrmb:partprio ---
	SavePartPrio();
	//--- :xrmb ---
	m_bInSave	=	false;		//Cax2 - bugfix for all those problems!!! :P
	EMULE_CATCH
}
 
void CKnownFileList::Clear()
{
	EMULE_TRY

	for (int i = 0; i < GetCount();i++)
		safe_delete(ElementAt(i));
	RemoveAll();
	SetSize(0);

	EMULE_CATCH
}

CKnownFile* CKnownFileList::FindKnownFile(char* filename,uint32 in_date,uint32 in_size)
{
	EMULE_TRY

	for (int i = 0;i < GetCount();i++){
		if (ElementAt(i)->GetFileDate() == in_date && ElementAt(i)->GetFileSize() == in_size && (!strcmp(filename,ElementAt(i)->GetFileName())))
			return ElementAt(i);
	}

	EMULE_CATCH

	return 0;
}

void CKnownFileList::SafeAddKFile(CKnownFile* toadd)
{
	CSingleLock sLock(&list_mut,true);
	Add(toadd);
	sLock.Unlock();
}

//--- xrmb:statsautosave ---
//Cax2 - totally unnecessary now that save is  called from a thread every 30 minutes!!!!
void CKnownFileList::Process()
{
	EMULE_TRY

		Save();

	EMULE_CATCH
}
//--- :xrmb ---

//--- xrmb:allyaknow ---
bool CKnownFileList::merge()
{
	EMULE_TRY

	CSingleLock sLock(&list_mut,true);

	int totalcnt=0;
	int items=GetCount();

	//--- go thru all items ---
	for(int i=0; i<items; i++)
	{
		int cnt=0;

		//--- compare with all other items ---
		for(int j=i+1; j<items; j++)
		{
			//--- same id? and same size... maybe we should compare the hashsets too? ---
			if(	ElementAt(i)->GetFileSize()==ElementAt(j)->GetFileSize() &&
				memcmp(ElementAt(i)->GetFileHash(), ElementAt(j)->GetFileHash(), 16)==0
				)
			{
				AddLogLine(false, "merge: '%s' and '%s'.", ElementAt(i)->GetFileName(), ElementAt(j)->GetFileName());

				//--- one file can be shared ---
				if(ElementAt(i)->GetSharedFile() && ElementAt(j)->GetSharedFile())
				{
					AddLogLine(false, "merge: you share the same file twice? abort merging.");
					sLock.Unlock();
					return false;
				}

				//--- if second file is shared, swap them ---
				if(ElementAt(j)->GetSharedFile())
				{
					AddLogLine(false, "merge: swapping files.");

					CKnownFile *t=ElementAt(i);
					SetAt(i, ElementAt(j));
					SetAt(j, t);
				}

				//--- merge into first item ---
				if(ElementAt(i)->statistic.merge(&ElementAt(j)->statistic)==false)
				{
					AddLogLine(false, "merge: failed.");
					sLock.Unlock();
					return false;
				}

				//--- remove second item ---
				CKnownFile	*d=ElementAt(j);
				RemoveAt(j);
				delete d;
				items--;
				j--;

				cnt++;
				totalcnt++;
			}
		}

		if(cnt)
		{
			AddLogLine(false, "merge: item %i merged with %i other items.", i, cnt);
		}
	}

	AddLogLine(false, "merge: %i items merged in total.", totalcnt);

	sLock.Unlock();
	return true;

	EMULE_CATCH

	return false;
}

bool CKnownFileList::RemoveFile(CKnownFile* toRemove)
{
	EMULE_TRY

	CSingleLock sLock(&list_mut, true);

	//--- go thru all items ---
	for(int i=0; i<GetCount(); i++)
	{
		if(ElementAt(i)==toRemove)
		{
			RemoveAt(i);
			delete toRemove;
		
			sLock.Unlock();
			return true;
		}
	}

	sLock.Unlock();

	EMULE_CATCH

	return false;
}

//--- xrmb:partprio ---
bool	CKnownFileList::LoadPartPrio()
{
	//--- save the part-permission ---
	FILE	*file=fopen(partPermissionFilePath, "rb");
	if(file==NULL)
		return false;

	uint8 version=1;
	fread(&version, sizeof(version), 1, file);	

	uint32	RecordsNumber=0;
	fread(&RecordsNumber, sizeof(RecordsNumber), 1, file);

	CKnownFile	*kf;
	uchar		hash[16];
	uint32		filesize;
	uint32		fpos;
	for(uint32 i=0; i<RecordsNumber; i++) 
	{
		//--- read hash ---
		fread(hash, 16, 1, file);

		//--- read filesize ---
		fread(&filesize, sizeof(filesize), 1, file);

		//--- save pos ---
		fpos=ftell(file);
		
		//--- find all these files ---
		for(int j=0; j<GetCount(); j++)
		{
			kf=ElementAt(j);

			//--- right file? ---
			if(kf->GetFileSize()==filesize && memcmp(kf->GetFileHash(), hash, 16)==0)
			{
				fseek(file, fpos, SEEK_SET);

				for(uint16 part=0; part<kf->GetPartCount(); part++)
				{
					uint8	state=PR_PART_ON;
					fread(&state, sizeof(state), 1, file);
					kf->SetPartStatus(part, state);
				}
			}
		}
	}
	fclose(file);

	return true;
}

bool	CKnownFileList::SavePartPrio()
{
	//--- save the part-permission ---
	FILE	*file=fopen(partPermissionFilePath, "wb");
	if(file==NULL)
		return false;

	uint8 version=1;
	fwrite(&version, sizeof(version), 1, file);	

	//--- preserve the space for the recordnumers ---
	uint32	RecordsNumber=0;
	uint32	rnpos=ftell(file);
	fwrite(&RecordsNumber, sizeof(RecordsNumber), 1, file);
	for(int i=0; i<GetCount(); i++) 
	{
		CKnownFile	*kf=ElementAt(i);

		// only save for known-files with traffic
		if(kf->HasHiddenParts())
		{
			//--- write hash ---
			fwrite(kf->GetFileHash(), 16, 1, file);

			//--- write filesize ---
			uint32 fs=kf->GetFileSize();
			fwrite(&fs, sizeof(fs), 1, file);
			
			//--- save prio ---
			for(uint16 part=0; part<kf->GetPartCount(); part++)
			{
				uint8	state=kf->GetPartStatus(part);
				fwrite(&state, sizeof(state), 1, file);
			}

			RecordsNumber++;
		}
	}

	//--- save the number of stored records ---
	fseek(file, rnpos, SEEK_SET);
	fwrite(&RecordsNumber, sizeof(RecordsNumber), 1, file);

	fclose(file);

	return true;
}
//--- :xrmb/partprio ---

//--- xrmb:parttraffic ---
bool	CKnownFileList::LoadPartTraffic()
{
	//--- load collected traffic data in our file ---
	FILE	*fh=fopen(trafficFilePath, "rb");
	if(!fh)
		return false;

	uint8	version=0;
	fread(&version, sizeof(version), 1, fh);

	uint32	records;
	fread(&records, sizeof(records), 1, fh);

	uchar tmp_filehash[16];
	uint32 fs=0;
	for(uint32 i=0; i<records; i++) 
	{
		uint8 hasData=0;
		fread(&hasData, sizeof(hasData), 1, fh);

		if(hasData)
		{
			fread(tmp_filehash, 16, 1, fh);
			fread(&fs, sizeof(fs), 1, fh);
			if(fs==ElementAt(i)->GetFileSize() && memcmp(tmp_filehash, ElementAt(i)->GetFileHash(), 16)==0)
				ElementAt(i)->LoadFromFileTraffic(fh, version);
			else
			{
				AddLogLine(false, "'traffic.dat' and 'known.met' aren't synchronous after %d datasets.", i);
				break;
			}
		}
	}
	fclose(fh);

	return true;
}

bool	CKnownFileList::SavePartTraffic()
{
	//--- save traffic data to our file ---

	FILE	*file=fopen(trafficFilePath, "wb");
	if(file==NULL)
		return false;

	uint8	version=2;
	fwrite(&version, sizeof(version), 1, file);	

	uint32	RecordsNumber=GetCount();
	fwrite(&RecordsNumber, sizeof(RecordsNumber), 1, file);
	for(uint32 i=0; i<RecordsNumber; i++) 
	{
		// only save for known-files with traffic
		if(ElementAt(i)->statistic.partTraffic)
		{
			uint8 hasData=1;
			fwrite(&hasData, sizeof(hasData), 1, file);

			//--- write hash ---
			fwrite(ElementAt(i)->GetFileHash(), 16, 1, file);

			//--- write filesize ---
			uint32 fs=ElementAt(i)->GetFileSize();
			fwrite(&fs, sizeof(fs), 1, file);
			
			//--- write part/block traffic ---
			ElementAt(i)->SaveToFileTraffic(file);
		}
		else
		{
			uint8 hasData=0;
			fwrite(&hasData, sizeof(hasData), 1, file);
		}
	}

	fclose(file);

	return true;
}
//--- :xrmb/parttraffic ---
